#include "MainWindow.hpp"
#include "Utility.hpp"
#include "CommonDef.hpp"
#include "sfyguiapp.brh"

#define X_MARGIN                5
#define Y_MARGIN                5
#define LINE_HEIGHT             (SFXGraphics::GetInstance()->GetFontHeight() + 4)
#define FONT_WIDTH              15
#define LABEL_TITLE             (LoadString(SFYGUIAPP_RES_FILE, IDS_MAIN_TITLE))
#define LABEL_YEAR              (LoadString(SFYGUIAPP_RES_FILE, IDS_YEAR))
#define LABEL_MONTH             (LoadString(SFYGUIAPP_RES_FILE, IDS_MONTH))
#define LABEL_DAY               (LoadString(SFYGUIAPP_RES_FILE, IDS_DAY))
#define LABEL_PLAN              (LoadString(SFYGUIAPP_RES_FILE, IDS_PLAN))
#define PLAN_ANUAL_MEETING      (LoadString(SFYGUIAPP_RES_FILE, IDS_ANUAL))
#define PLAN_SPECIAL_MEETING    (LoadString(SFYGUIAPP_RES_FILE, IDS_SPECIAL))
#define PLAN_BUSSINESS_TRIP     (LoadString(SFYGUIAPP_RES_FILE, IDS_BUSSINESS_TRIP))
#define PLAN_LUNCH              (LoadString(SFYGUIAPP_RES_FILE, IDS_LUNCH))
#define PLAN_DRINKING           (LoadString(SFYGUIAPP_RES_FILE, IDS_DRINKING))
#define PLAN_BIRTHDAY_PARTY     (LoadString(SFYGUIAPP_RES_FILE, IDS_BIRTHDAY_PARTY))
#define PLAN_WEDDING            (LoadString(SFYGUIAPP_RES_FILE, IDS_WEDDING))
#define PLAN_SECRET             (LoadString(SFYGUIAPP_RES_FILE, IDS_SECRET))
#define PLAN_VERY_LONG          (LoadString(SFYGUIAPP_RES_FILE, IDS_LONGSTRING))
#define LABEL_DETAIL            (LoadString(SFYGUIAPP_RES_FILE, IDS_DETAIL))
#define LABEL_PRIORITY          (LoadString(SFYGUIAPP_RES_FILE, IDS_PRIORITY))
#define LABEL_PRIORITY_HIGH     (LoadString(SFYGUIAPP_RES_FILE, IDS_HIGH))
#define LABEL_PRIORITY_MIDDLE   (LoadString(SFYGUIAPP_RES_FILE, IDS_MIDDLE))
#define LABEL_PRIORITY_LOW      (LoadString(SFYGUIAPP_RES_FILE, IDS_LOW))
#define LABEL_ALARM             (LoadString(SFYGUIAPP_RES_FILE, IDS_ALARM))
#define LABEL_CREATE            (LoadString(SFYGUIAPP_RES_FILE, IDS_CREATE))
#define LABEL_CANCEL            (LoadString(SFYGUIAPP_RES_FILE, IDS_CANCEL))
#define MSG_CONFIRM_EXIT        (LoadString(SFYGUIAPP_RES_FILE, IDS_CONFIRM_EXIT))
#define ITEM_CONFIG             (LoadString(SFYGUIAPP_RES_FILE, IDS_MENU_CONFIG))
#define ITEM_VERSION            (LoadString(SFYGUIAPP_RES_FILE, IDS_MENU_VERSION))
#define LABEL_OK                (LoadString(SFYGUIAPP_RES_FILE, IDS_OK))
#define LABEL_CANCEL            (LoadString(SFYGUIAPP_RES_FILE, IDS_CANCEL))
#define LABEL_EXIT_TITLE        (LoadString(SFYGUIAPP_RES_FILE, IDS_EXIT_TITLE))
#define COLOR_NEWWINDOW         COLOR_LIGHT_PURPLE
#define COLOR_NEWTITLE          COLOR_DEEP_PURPLE
#define COLOR_NEWTITLE_FONT     COLOR_WHITE
#define COLOR_MENU_TITLE_BACK   COLOR_LIGHT_GREEN
#define COLOR_MENU_TITLE_FONT   COLOR_WHITE
#define COLOR_MENU_SELBACK      COLOR_LIGHT_PINK
#define COLOR_MENU_ITEMBACK     COLOR_LIGHT_PURPLE
#define COLOR_TAB               COLOR_BLACK
#define ATOM_PAGE_ONE           {0x00, 0xFD, 0xCC, 0x8A}
#define ATOM_PAGE_TWO           {0x00, 0xB2, 0x9E, 0x93}
#define ATOM_PAGE_THREE         {0x00, 0xCC, 0xD4, 0xC7}

// RXgN^
MainWindow::MainWindow(Void) static_throws
{
    static SFXEventRange::AtomRecConst range[] = {
        {SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, AVK_SOFT1, AVK_SOFT2}
    };
    SFYHandler::RuleRec rule[lengthof(range)];

    if (static_try()) {
        SetType(CODE_TYPE);
        rule[0].spp = XANDLER_FUNCTION(OnSoft);
        rule[0].reference = this;
        static_throw(RegisterHandler(atomic_cast(range), rule, lengthof(range)));
        if (static_try()) {
            if ((_soft = GetSoftKey()) == null) {
                static_throw(SFERR_FAILED);
            }
        }
    }
}

// fXgN^
MainWindow::~MainWindow(Void)
{
}

// CX^X쐬
MainWindowSmp MainWindow::NewInstance(SFCErrorPtr exception)
{
    return static_pointer_cast<MainWindow>(Factory(:: new MainWindow, exception));
}

// X|_쐬
SFCError MainWindow::Make(Void)
{
    SFXGrid offset(X_MARGIN, Y_MARGIN);
    SFCError error;

    if ((error = CreateDate(&offset)) != SFERR_NO_ERROR) {
        TRACE("Error in Date Line");
        return error;
    }
    if ((error = CreatePlan(&offset)) != SFERR_NO_ERROR) {
        TRACE("Error in Plan Line");
        return error;
    }
    if ((error = CreateDetail(&offset)) != SFERR_NO_ERROR) {
        TRACE("Error in Detail Line");
        return error;
    }
    if ((error = CreatePriority(&offset)) != SFERR_NO_ERROR) {
        TRACE("Error in Priority Line");
        return error;
    }
    if ((error = CreateAlarm(&offset)) != SFERR_NO_ERROR) {
        TRACE("Error in Alarm Line");
        return error;
    }
    if ((error = CreateButtons(&offset)) != SFERR_NO_ERROR) {
        TRACE("Error in Buttons Line");
        return error;
    }
    return error;
}

// t̍쐬Ɣzu(xƃGfBbg{bNX)
SFCError MainWindow::CreateDate(SFXGridPtr offs)
{
    SFCError error;
    SFXGrid initial = *offs;
    SFXRectangle rect;
    SFXDate today = SFXDate::CurrentDate();

    if ((_edtYear = SFZSingleEditBoxControl::NewInstance(&error)) != null) {
        error = _edtYear->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _edtYear->SetText(SFXWideString::Format("%04d", today.GetYear()));
            if (error == SFERR_NO_ERROR) {
                error = _edtYear->SetMaximumLength(4);
                if (error == SFERR_NO_ERROR) {
                    _edtYear->SetInputMode(AEE_TM_NUMBERS);
                    rect = SFXRectangle(*offs, _edtYear->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                    _edtYear->SetRealBound(rect);
                    _edtYear->SetState(true, true, true, false);
                }
            }
        }
    }
    offs->AddX(_edtYear->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_lblYear = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        error = _lblYear->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _lblYear->SetText(LABEL_YEAR);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _lblYear->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _lblYear->SetRealBound(rect);
                _lblYear->SetState(true, true, false, false);
            }
        }
    }
    offs->AddX(_lblYear->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_edtMonth = SFZSingleEditBoxControl::NewInstance(&error)) != null) {
        error = _edtMonth->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _edtMonth->SetText(SFXWideString::Format("%02d", today.GetMonth()));
            if (error == SFERR_NO_ERROR) {
                error = _edtMonth->SetMaximumLength(2);
                if (error == SFERR_NO_ERROR) {
                    _edtMonth->SetInputMode(AEE_TM_NUMBERS);
                    rect = SFXRectangle(*offs, FONT_WIDTH * 2, LINE_HEIGHT);
                    _edtMonth->SetRealBound(rect);
                    _edtMonth->SetState(true, true, true, false);
                }
            }
        }
    }
    offs->AddX(_edtMonth->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_lblMonth = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        error = _lblMonth->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _lblMonth->SetText(LABEL_MONTH);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _lblMonth->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _lblMonth->SetRealBound(rect);
                _lblMonth->SetState(true, true, false, false);
            }
        }
    }
    offs->AddX(_lblMonth->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_edtDay = SFZSingleEditBoxControl::NewInstance(&error)) != null) {
        error = _edtDay->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _edtDay->SetText(SFXWideString::Format("%02d", today.GetDay()));
            if (error == SFERR_NO_ERROR) {
                error = _edtDay->SetMaximumLength(2);
                if (error == SFERR_NO_ERROR) {
                    _edtDay->SetInputMode(AEE_TM_NUMBERS);
                    rect = SFXRectangle(*offs, FONT_WIDTH * 2, LINE_HEIGHT);
                    _edtDay->SetRealBound(rect);
                    _edtDay->SetState(true, true, true, false);
                }
            }
        }
    }
    offs->AddX(_edtDay->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_lblDay = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        error = _lblDay->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _lblDay->SetText(LABEL_DAY);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _lblDay->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _lblDay->SetRealBound(rect);
                _lblDay->SetState(true, true, false, false);
            }
        }
    }
    offs->Set(initial.GetX(), initial.GetY() + LINE_HEIGHT + Y_MARGIN);
    return error;
}

// \̍쐬Ɣzu(xƃR{{bNX)
SFCError MainWindow::CreatePlan(SFXGridPtr offs)
{
    SFCError error;
    SFXGrid initial = *offs;
    SFXRectangle rect;

    if ((_lblPlan = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        error = _lblPlan->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _lblPlan->SetText(LABEL_PLAN);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _lblPlan->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _lblPlan->SetRealBound(rect);
                _lblPlan->SetState(true, true, false, false);
            }
        }
    }
    offs->AddX(_lblPlan->GetLocalBound().GetWidth() + X_MARGIN);

    if ((_cmbPlan = SFZComboBoxControl::NewInstance(&error)) != null) {
        error = _cmbPlan->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            SFXWideString item[] = {
                PLAN_ANUAL_MEETING, PLAN_SPECIAL_MEETING, PLAN_BUSSINESS_TRIP,PLAN_LUNCH,
                PLAN_DRINKING,      PLAN_BIRTHDAY_PARTY,  PLAN_WEDDING,       PLAN_SECRET,
                PLAN_VERY_LONG
            };
            for (int i = 0; i < sizeof(item) / sizeof(item[0]); ++i) {
                error = _cmbPlan->Append(item[i]);
                if (error != SFERR_NO_ERROR) {
                    TRACE("item error");
                    break;
                }
            }
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, GetLocalBound().GetWidth(), LINE_HEIGHT);
                rect.SetRight(GetLocalBound().GetWidth() - X_MARGIN);
                _cmbPlan->SetRealBound(rect);
                _cmbPlan->SetState(true, true, true, false);
            }
        }
    }
    offs->Set(initial.GetX(), initial.GetY() + LINE_HEIGHT + Y_MARGIN);

    return error;
}

// ڍׂ̍쐬Ɣzu(xƕsGfBbg{bNX)
SFCError MainWindow::CreateDetail(SFXGridPtr offs)
{
    SFCError error;
    SFXGrid initial = *offs;
    SFXRectangle rect;

    if ((_lblDetail = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        error = _lblDetail->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _lblDetail->SetText(LABEL_DETAIL);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _lblDetail->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _lblDetail->SetRealBound(rect);
                _lblDetail->SetState(true, true, false, false);
            }
        }
    }
    offs->AddX(_lblDetail->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_edtDetail = SFZMultipleEditBoxControl::NewInstance(&error)) != null) {
        error = _edtDetail->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            rect = SFXRectangle(*offs, GetLocalBound().GetWidth(), LINE_HEIGHT * 3);
            rect.SetRight(GetLocalBound().GetWidth() - X_MARGIN);
            _edtDetail->SetRealBound(rect);
            _edtDetail->SetHorizontalAlign(SFZMultipleEditBoxControl::HORIZONTAL_LEFT);
            _edtDetail->SetVerticalAlign(SFZMultipleEditBoxControl::VERTICAL_TOP);
            _edtDetail->SetState(true, true, true, false);
        }
    }
    offs->Set(initial.GetX(), initial.GetY() + LINE_HEIGHT * 3 + Y_MARGIN);

    return error;
}

// dvx̍쐬Ɣzu(xƃWI{^)
SFCError MainWindow::CreatePriority(SFXGridPtr offs)
{
    SFCError error;
    SFXGrid initial = *offs;
    SFXRectangle rect;

    if ((_lblPriority = SFZSingleTextLabelControl::NewInstance(&error)) != null) {
        error = _lblPriority->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _lblPriority->SetText(LABEL_PRIORITY);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _lblPriority->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _lblPriority->SetRealBound(rect);
                _lblPriority->SetState(true, true, false, false);
            }
        }
    }
    offs->AddX(_lblPriority->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_rdoHigh = SFZRadiobuttonControl::NewInstance(&error)) != null) {
        error = _rdoHigh->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _rdoHigh->SetText(LABEL_PRIORITY_HIGH);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _rdoHigh->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _rdoHigh->SetCurrentValue(true);
                _rdoHigh->SetRealBound(rect);
                _rdoHigh->SetState(true, true, true, false);
            }
        }
    }
    offs->AddX(_rdoHigh->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_rdoMiddle = SFZRadiobuttonControl::NewInstance(&error)) != null) {
        error = _rdoMiddle->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _rdoMiddle->SetText(LABEL_PRIORITY_MIDDLE);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _rdoMiddle->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _rdoMiddle->SetRealBound(rect);
                _rdoMiddle->SetState(true, true, true, false);
            }
        }
    }
    offs->AddX(_rdoMiddle->GetLocalBound().GetWidth() + X_MARGIN);
    if ((_rdoLow = SFZRadiobuttonControl::NewInstance(&error)) != null) {
        error = _rdoLow->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _rdoLow->SetText(LABEL_PRIORITY_LOW);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _rdoLow->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _rdoLow->SetRealBound(rect);
                _rdoLow->SetState(true, true, true, false);
            }
        }
    }
    _rdoHigh->Group(_rdoMiddle);
    _rdoHigh->Group(_rdoLow);
    _rdoMiddle->Group(_rdoLow);
    offs->Set(initial.GetX(), initial.GetY() + LINE_HEIGHT + Y_MARGIN);
    return error;
}

// A[̍쐬Ɣzu(`FbN{bNX)
SFCError MainWindow::CreateAlarm(SFXGridPtr offs)
{
    SFCError error;
    SFXGrid initial = *offs;
    SFXRectangle rect;

    if ((_chkAlarm = SFZCheckboxControl::NewInstance(&error)) != null) {
        error = _chkAlarm->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _chkAlarm->SetText(LABEL_ALARM);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, _chkAlarm->GetSuitableBound().GetWidth(), LINE_HEIGHT);
                _chkAlarm->SetRealBound(rect);
                _chkAlarm->SetState(true, true, true, false);
            }
        }
    }
    offs->Set(initial.GetX(), initial.GetY() + LINE_HEIGHT + Y_MARGIN);
    return error;
}

// {^̍쐬Ɣzu
SFCError MainWindow::CreateButtons(SFXGridPtr offs)
{
    SFCError error;
    SFXGrid initial = *offs;
    SFXRectangle rect;
    SInt16 width = (GetLocalBound().GetWidth() - X_MARGIN * 3) / 2;

    if ((_btnCreate = SFZTextButtonControl::NewInstance(&error)) != null) {
        error = _btnCreate->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _btnCreate->SetText(LABEL_CREATE);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, width, LINE_HEIGHT);
                _btnCreate->SetRealBound(rect);
                _btnCreate->SetState(true, true, true, false);
            }
        }
    }
    offs->AddX(width + X_MARGIN);
    if ((_btnCancel = SFZTextButtonControl::NewInstance(&error)) != null) {
        error = _btnCancel->SetParent(GetThis());
        if (error == SFERR_NO_ERROR) {
            error = _btnCancel->SetText(LABEL_CANCEL);
            if (error == SFERR_NO_ERROR) {
                rect = SFXRectangle(*offs, width, LINE_HEIGHT);
                _btnCancel->SetRealBound(rect);
                _btnCancel->SetState(true, true, true, false);
            }
        }
    }
    offs->Set(initial.GetX(), initial.GetY() + LINE_HEIGHT + Y_MARGIN);
    return error;
}

// I_CAO̍쐬Ɣzu
SFCError MainWindow::MakeExitDialog(Void)
{
    SFZTitleFlatFrameSmp frame;
    SFZQuestionDialogSmp dialog;
    SFCError error;

    if ((frame = SFZTitleFlatFrame::NewInstance(&error)) != null) {

        // t[̐e̓[gɂ
        SFYResponderSmp root = SFYApplication::GetRoot();

        if ((error = frame->SetParent(root)) == SFERR_NO_ERROR) {
            if ((dialog = SFZQuestionDialog::NewInstance(&error)) != null) {

                // \tgL[Ɗ֘At
                if ((error = _soft->Bind(dialog.Get(), SOFTMENU_SELECT)) == SFERR_NO_ERROR) {

                    if ((error = dialog->SetParent(frame)) == SFERR_NO_ERROR) {

                        // _CAǑʃnho^
                        error = dialog->RegisterHandler(
                            SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                            XANDLER_INTERNAL(OnExitDialogResult)
                        );

                        if (error == SFERR_NO_ERROR) {
                            if ((error = dialog->SetMessageText(MSG_CONFIRM_EXIT)) == SFERR_NO_ERROR) {
                                if ((error = dialog->SetButtonText(SFZQuestionDialog::BUTTON_OK, LABEL_OK)) == SFERR_NO_ERROR) {
                                    if ((error = dialog->SetButtonText(SFZQuestionDialog::BUTTON_CANCEL, LABEL_CANCEL)) == SFERR_NO_ERROR) {
                                        frame->SetText(LABEL_EXIT_TITLE);

                                        // _CAOgщʒɔzu
                                        SFXMargin margin(frame->GetSuitableMargin());
                                        SFXRectangle rect(dialog->GetSuitableBound(root->GetLocalBound().Deflate(10, 10).Deflate(margin)));
                                        rect.Inflate(margin);
                                        rect.SetOrigin(SFXGrid(root->GetLocalBound().GetSize() - rect.GetSize()) / 2);
                                        frame->SetRealBound(rect);
                                        dialog->SetRealBound(frame->GetLocalBound().Deflate(margin));

                                        frame->SetState(true, true, true, true);
                                        dialog->SetState(true, true, true, true);
                                        frame->ToFront();
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return error;
}

// ݒ胁j[̍쐬Ɣzu
SFCError MainWindow::MakeConfigMenu(Void)
{
    static SFXEventRange::AtomRecConst range[] = {
        {SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END}
    };
    SFYHandler::RuleRec rule[lengthof(range)];
    SFZTextMenuSmp menu;
    SFCError error;

    if ((menu = SFZTextMenu::NewInstance(&error)) != null) {

        // \tgL[Ɗ֘At
        if ((error = _soft->Bind(menu.Get(), SOFTMENU_SELECT)) == SFERR_NO_ERROR) {

            if ((error = menu->SetParent(GetThis())) == SFERR_NO_ERROR) {

                // j[̌ʃnho^
                rule[0].spp = XANDLER_FUNCTION(OnConfigMenuResult);
                rule[0].reference = this;
                if ((error = menu->RegisterHandler(atomic_cast(range), rule, lengthof(range))) == SFERR_NO_ERROR) {

                    if ((error = menu->SetTitle("ƭ")) == SFERR_NO_ERROR) {

                        // j[̔zFݒ
                        SFXBevelColor menuBevel(COLOR_LIGHT_PURPLE, COLOR_DEEP_PURPLE, COLOR_BLACK);
                        menu->SetTitleForeColor(COLOR_MENU_TITLE_FONT);
                        menu->SetTitleBackColor(COLOR_MENU_TITLE_BACK);
                        menu->SetSelBackColor(COLOR_MENU_SELBACK);
                        menu->SetBevelColor(menuBevel);

                        // j[ɂQڒǉ
                        if ((error = menu->AppendItem(ITEM_CONFIG, AVK_1, 0xFBF6)) == SFERR_NO_ERROR) {
                            if ((error = menu->AppendItem(ITEM_VERSION, AVK_2, 0xFCF6)) == SFERR_NO_ERROR) {
                                SFXRectangle rect(GetLocalBound());
                                rect.SetLeftTop(rect.GetWidth()/4, rect.GetBottom() - LINE_HEIGHT * 3);
                                menu->SetRealBound(rect);
                                menu->SetState(true, true, true, true);
                                menu->ToFront();
                            }
                        }
                    }
                }
            }
        }
    }
    return error;
}

// ^uRg[zuEBhE̍쐬
SFCError MainWindow::MakeTabWindow(Void)
{
    SFXRGBColor::AtomRecConst color[3] = {
        ATOM_PAGE_ONE, ATOM_PAGE_TWO, ATOM_PAGE_THREE
    };
    SFZWindowSmp window;
    SFZTabControlSmp tab;
    SFZTabPageSmp page[3];
    SFCError error(SFERR_NO_ERROR);

    // EBhE쐬
    if ((window = SFZWindow::NewInstance(&error)) != null) {

        // EBhE̐e[gɐݒ
        if ((error = window->SetParent(SFYApplication::GetRoot())) == SFERR_NO_ERROR) {

            // \tgL[Ɗ֘At
            if ((error = _soft->Bind(window.Get(), SOFTMENU_CONFIG)) == SFERR_NO_ERROR) {

                // \tgL[nhEBhEɓo^
                error = window->RegisterHandler(
                    SFXEventRange(SFEVT_RESPONDER_SOFTKEY, SFEVT_RESPONDER_SOFTKEY, SFP16_BEGIN, SFP16_END),
                    XANDLER_INTERNAL(OnTabWindowSoftKey)
                );

                if (error == SFERR_NO_ERROR) {

                    // EBhË̗gт̉ʂ\tgL[Rg[̈ɐݒ
                    window->SetRealBound(SFYApplication::GetRoot()->GetLocalBound().SetBottom(_soft->GetRealBound().GetTop()));

                    window->SetState(true, true, true, true);

                    // EBhEOʂɏo
                    window->ToFront();

                    // ^uRg[̍쐬
                    if ((tab = SFZTabControl::NewInstance(&error)) != null) {

                        // ^uRg[̐eEBhEɐݒ
                        if ((error = tab->SetParent(window)) == SFERR_NO_ERROR) {

                            SFXBevelColor tabBevel(COLOR_LIGHT_PURPLE, COLOR_DEEP_PURPLE, COLOR_BLACK);

                            tab->SetRealBound(window->GetLocalBound());
                            tab->SetBackgroundColor(COLOR_TAB);

                            // ^uRg[̃pfBORɐݒ
                            tab->SetPadding(3);

                            tab->SetArrowColor(COLOR_BLACK);
                            tab->SetTabBevelColor(tabBevel);
                            tab->SetDrawBorder(true);

                            // xɂR^u\
                            tab->SetFieldValue(3);

                            tab->SetState(true, true, true, true);
                            for (SIntN i = 0; i < 3; ++i) {
                                if ((page[i] = SFZTabPage::NewInstance(&error)) != null) {

                                    // y[W^uɒǉƃy[W͎̎̈Iɐݒ肳
                                    if ((error = page[i]->SetParent(tab)) == SFERR_NO_ERROR) {

                                        // ^uy[W̃^Cg
                                        SFXWideString title;
                                        page[i]->SetTitle((title = LoadString(SFYGUIAPP_RES_FILE, IDS_PAGE1_TITLE + i * 2)));

                                        // ^uy[W̃qg
                                        page[i]->SetHint(LoadString(SFYGUIAPP_RES_FILE, IDS_PAGE1_HINT + i * 2));

                                        page[i]->SetBackgroundColor(color[i]);

                                        // y[W̉z̈ 100px ɍL
                                        page[i]->SetVirtualBound(page[i]->GetLocalBound().Inflate(0, 0, 0, 100));

                                        page[i]->SetState(true, true, true, false);

                                        // y[W̒̃{^쐬
                                        SFZTextButtonControlSmp button = SFZTextButtonControl::NewInstance(&error);

                                        if (button != null) {

                                            // {^̐ey[Wɂ
                                            if ((error = button->SetParent(page[i])) == SFERR_NO_ERROR) {
                                                SFXRectangle rx;

                                                button->SetText(title);

                                                // {^̗̈ݒ肷
                                                rx.Set(button->GetSuitableBound(page[i]->GetLocalBound()));
                                                rx.SnapCenter(page[i]->GetLocalBound().GetCenter());
                                                rx.SnapBottom(page[i]->GetLocalBound().GetBottom() / (3 - i));
                                                button->SetRealBound(rx);

                                                // {^Ԃݒ肷
                                                button->SetState(true, true, true, true);
                                            }
                                        }
                                    }
                                }
                                if (error != SFERR_NO_ERROR) {
                                    break;
                                }
                            }
                            if (error == SFERR_NO_ERROR) {
                                // Pڂ̃^uy[W\
                                tab->FocusPage(0);
                            }
                        }
                    }
                }
            }
        }
    }
    return error;
}

// o[W_CAO̍쐬Ɣzu
SFCError MainWindow::MakeVersionDialog(Void)
{
    SFZTitleFlatFrameSmp frame;
    SFZMessageDialogSmp dialog;
    SFCError error(SFERR_NO_ERROR);

    if ((frame = SFZTitleFlatFrame::NewInstance(&error)) != null) {
        SFYResponderSmp root = SFYApplication::GetRoot();
        if ((error = frame->SetParent(root)) == SFERR_NO_ERROR) {
            if ((dialog = SFZMessageDialog::NewInstance(&error)) != null) {

                // \tgL[Ɗ֘At
                if ((error =_soft->Bind(dialog.Get(), SOFTMENU_SELECT)) == SFERR_NO_ERROR) {

                    if ((error = dialog->SetParent(frame)) == SFERR_NO_ERROR) {
                        error = dialog->RegisterHandler(
                                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                                XANDLER_INTERNAL(OnVersionDlgResult)
                        );
                        if (error == SFERR_NO_ERROR) {

                            // _CAOɃC[Wݒ
                            if ((error = dialog->SetIconImage(SFXPath(SFYGUIAPP_RES_FILE), VERSION)) == SFERR_NO_ERROR) {

                                if ((error = dialog->SetButtonText("OK")) == SFERR_NO_ERROR) {
                                    frame->SetText(ITEM_VERSION);

                                    // _CAOgщʒɔzu
                                    SFXMargin margin(frame->GetSuitableMargin());
                                    SFXRectangle rect(dialog->GetSuitableBound(root->GetLocalBound().Deflate(10, 10).Deflate(margin)));
                                    rect.Inflate(margin);
                                    rect.SetOrigin(SFXGrid(root->GetLocalBound().GetSize() - rect.GetSize()) / 2);
                                    frame->SetRealBound(rect);
                                    dialog->SetRealBound(frame->GetLocalBound().Deflate(margin));

                                    frame->SetState(true, true, true, true);
                                    dialog->SetState(true, true, true, true);
                                    frame->ToFront();

                                    // 3000 ~bɃ_CAO
                                    dialog->ScheduleTimer(3000);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return error;
}

// CEBhẼ\tgL[nh
XANDLER_IMPLEMENT_VOIDEVENT(MainWindow, OnSoft, invoker, event)
{
    unused(invoker);

    switch(event.GetP16()) {
        case AVK_SOFT1:
            MakeExitDialog();   // I_CAO
            break;
        case AVK_SOFT2:
            MakeConfigMenu();   // ݒ胁j[
            break;
    }
    return;
}

// I_CAȎI
XANDLER_IMPLEMENT_VOIDRESULT(MainWindow, OnExitDialogResult, invoker, reason, result)
{
    unused(result);

    if (reason == SFP16_RESULT_OK) {

        // AvI
        SFYApplication::Terminate();

    }
    else {

        // _CAO̐ełt[ƕ
        invoker->GetParent()->Terminate();

    }
    return;
}

// ݒ胁j[̌
XANDLER_IMPLEMENT_VOIDRESULT(MainWindow, OnConfigMenuResult, invoker, reason, result)
{
    if (reason == SFP16_RESULT_OK) {
        switch(result) {
            case 0:
                MakeTabWindow();        // ^uRg[zuEBhE
                break;
            case 1:
                MakeVersionDialog();    // o[W_CAO
                break;
        }
    }
    // j[
    invoker->Terminate();
    return;
}

// o[W_CAǑʃnh
XANDLER_IMPLEMENT_VOIDRESULT(MainWindow, OnVersionDlgResult, invoker, reason, result)
{
    unused(reason);
    unused(result);

    // _CAO̐ełt[ƕ
    invoker->GetParent()->Terminate();
    return;
}

// ^uRg[zuEBhẼ\tgL[nh
XANDLER_IMPLEMENT_VOIDEVENT(MainWindow, OnTabWindowSoftKey, invoker, event)
{
    switch (event.GetP16()) {
        case AVK_SOFT1:
        case AVK_SOFT2:

            // EBhE
            invoker->Terminate();
    }
    return;
}
